/*
Copyright 2020 Rancher Labs, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by main. DO NOT EDIT.

package v3

import (
	"context"
	"time"

	clientset "github.com/rancher/rancher/pkg/wrangler/generated/clientset/versioned/typed/management.cattle.io/v3"
	informers "github.com/rancher/rancher/pkg/wrangler/generated/informers/externalversions/management.cattle.io/v3"
	listers "github.com/rancher/rancher/pkg/wrangler/generated/listers/management.cattle.io/v3"
	v3 "github.com/rancher/types/apis/management.cattle.io/v3"
	"github.com/rancher/wrangler/pkg/apply"
	"github.com/rancher/wrangler/pkg/condition"
	"github.com/rancher/wrangler/pkg/generic"
	"k8s.io/apimachinery/pkg/api/equality"
	"k8s.io/apimachinery/pkg/api/errors"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/labels"
	"k8s.io/apimachinery/pkg/runtime"
	"k8s.io/apimachinery/pkg/runtime/schema"
	"k8s.io/apimachinery/pkg/types"
	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
	"k8s.io/apimachinery/pkg/watch"
	"k8s.io/client-go/tools/cache"
)

type UserHandler func(string, *v3.User) (*v3.User, error)

type UserController interface {
	generic.ControllerMeta
	UserClient

	OnChange(ctx context.Context, name string, sync UserHandler)
	OnRemove(ctx context.Context, name string, sync UserHandler)
	Enqueue(name string)
	EnqueueAfter(name string, duration time.Duration)

	Cache() UserCache
}

type UserClient interface {
	Create(*v3.User) (*v3.User, error)
	Update(*v3.User) (*v3.User, error)
	UpdateStatus(*v3.User) (*v3.User, error)
	Delete(name string, options *metav1.DeleteOptions) error
	Get(name string, options metav1.GetOptions) (*v3.User, error)
	List(opts metav1.ListOptions) (*v3.UserList, error)
	Watch(opts metav1.ListOptions) (watch.Interface, error)
	Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v3.User, err error)
}

type UserCache interface {
	Get(name string) (*v3.User, error)
	List(selector labels.Selector) ([]*v3.User, error)

	AddIndexer(indexName string, indexer UserIndexer)
	GetByIndex(indexName, key string) ([]*v3.User, error)
}

type UserIndexer func(obj *v3.User) ([]string, error)

type userController struct {
	controllerManager *generic.ControllerManager
	clientGetter      clientset.UsersGetter
	informer          informers.UserInformer
	gvk               schema.GroupVersionKind
}

func NewUserController(gvk schema.GroupVersionKind, controllerManager *generic.ControllerManager, clientGetter clientset.UsersGetter, informer informers.UserInformer) UserController {
	return &userController{
		controllerManager: controllerManager,
		clientGetter:      clientGetter,
		informer:          informer,
		gvk:               gvk,
	}
}

func FromUserHandlerToHandler(sync UserHandler) generic.Handler {
	return func(key string, obj runtime.Object) (ret runtime.Object, err error) {
		var v *v3.User
		if obj == nil {
			v, err = sync(key, nil)
		} else {
			v, err = sync(key, obj.(*v3.User))
		}
		if v == nil {
			return nil, err
		}
		return v, err
	}
}

func (c *userController) Updater() generic.Updater {
	return func(obj runtime.Object) (runtime.Object, error) {
		newObj, err := c.Update(obj.(*v3.User))
		if newObj == nil {
			return nil, err
		}
		return newObj, err
	}
}

func UpdateUserDeepCopyOnChange(client UserClient, obj *v3.User, handler func(obj *v3.User) (*v3.User, error)) (*v3.User, error) {
	if obj == nil {
		return obj, nil
	}

	copyObj := obj.DeepCopy()
	newObj, err := handler(copyObj)
	if newObj != nil {
		copyObj = newObj
	}
	if obj.ResourceVersion == copyObj.ResourceVersion && !equality.Semantic.DeepEqual(obj, copyObj) {
		return client.Update(copyObj)
	}

	return copyObj, err
}

func (c *userController) AddGenericHandler(ctx context.Context, name string, handler generic.Handler) {
	c.controllerManager.AddHandler(ctx, c.gvk, c.informer.Informer(), name, handler)
}

func (c *userController) AddGenericRemoveHandler(ctx context.Context, name string, handler generic.Handler) {
	removeHandler := generic.NewRemoveHandler(name, c.Updater(), handler)
	c.controllerManager.AddHandler(ctx, c.gvk, c.informer.Informer(), name, removeHandler)
}

func (c *userController) OnChange(ctx context.Context, name string, sync UserHandler) {
	c.AddGenericHandler(ctx, name, FromUserHandlerToHandler(sync))
}

func (c *userController) OnRemove(ctx context.Context, name string, sync UserHandler) {
	removeHandler := generic.NewRemoveHandler(name, c.Updater(), FromUserHandlerToHandler(sync))
	c.AddGenericHandler(ctx, name, removeHandler)
}

func (c *userController) Enqueue(name string) {
	c.controllerManager.Enqueue(c.gvk, c.informer.Informer(), "", name)
}

func (c *userController) EnqueueAfter(name string, duration time.Duration) {
	c.controllerManager.EnqueueAfter(c.gvk, c.informer.Informer(), "", name, duration)
}

func (c *userController) Informer() cache.SharedIndexInformer {
	return c.informer.Informer()
}

func (c *userController) GroupVersionKind() schema.GroupVersionKind {
	return c.gvk
}

func (c *userController) Cache() UserCache {
	return &userCache{
		lister:  c.informer.Lister(),
		indexer: c.informer.Informer().GetIndexer(),
	}
}

func (c *userController) Create(obj *v3.User) (*v3.User, error) {
	return c.clientGetter.Users().Create(context.TODO(), obj, metav1.CreateOptions{})
}

func (c *userController) Update(obj *v3.User) (*v3.User, error) {
	return c.clientGetter.Users().Update(context.TODO(), obj, metav1.UpdateOptions{})
}

func (c *userController) UpdateStatus(obj *v3.User) (*v3.User, error) {
	return c.clientGetter.Users().UpdateStatus(context.TODO(), obj, metav1.UpdateOptions{})
}

func (c *userController) Delete(name string, options *metav1.DeleteOptions) error {
	if options == nil {
		options = &metav1.DeleteOptions{}
	}
	return c.clientGetter.Users().Delete(context.TODO(), name, *options)
}

func (c *userController) Get(name string, options metav1.GetOptions) (*v3.User, error) {
	return c.clientGetter.Users().Get(context.TODO(), name, options)
}

func (c *userController) List(opts metav1.ListOptions) (*v3.UserList, error) {
	return c.clientGetter.Users().List(context.TODO(), opts)
}

func (c *userController) Watch(opts metav1.ListOptions) (watch.Interface, error) {
	return c.clientGetter.Users().Watch(context.TODO(), opts)
}

func (c *userController) Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v3.User, err error) {
	return c.clientGetter.Users().Patch(context.TODO(), name, pt, data, metav1.PatchOptions{}, subresources...)
}

type userCache struct {
	lister  listers.UserLister
	indexer cache.Indexer
}

func (c *userCache) Get(name string) (*v3.User, error) {
	return c.lister.Get(name)
}

func (c *userCache) List(selector labels.Selector) ([]*v3.User, error) {
	return c.lister.List(selector)
}

func (c *userCache) AddIndexer(indexName string, indexer UserIndexer) {
	utilruntime.Must(c.indexer.AddIndexers(map[string]cache.IndexFunc{
		indexName: func(obj interface{}) (strings []string, e error) {
			return indexer(obj.(*v3.User))
		},
	}))
}

func (c *userCache) GetByIndex(indexName, key string) (result []*v3.User, err error) {
	objs, err := c.indexer.ByIndex(indexName, key)
	if err != nil {
		return nil, err
	}
	result = make([]*v3.User, 0, len(objs))
	for _, obj := range objs {
		result = append(result, obj.(*v3.User))
	}
	return result, nil
}

type UserStatusHandler func(obj *v3.User, status v3.UserStatus) (v3.UserStatus, error)

type UserGeneratingHandler func(obj *v3.User, status v3.UserStatus) ([]runtime.Object, v3.UserStatus, error)

func RegisterUserStatusHandler(ctx context.Context, controller UserController, condition condition.Cond, name string, handler UserStatusHandler) {
	statusHandler := &userStatusHandler{
		client:    controller,
		condition: condition,
		handler:   handler,
	}
	controller.AddGenericHandler(ctx, name, FromUserHandlerToHandler(statusHandler.sync))
}

func RegisterUserGeneratingHandler(ctx context.Context, controller UserController, apply apply.Apply,
	condition condition.Cond, name string, handler UserGeneratingHandler, opts *generic.GeneratingHandlerOptions) {
	statusHandler := &userGeneratingHandler{
		UserGeneratingHandler: handler,
		apply:                 apply,
		name:                  name,
		gvk:                   controller.GroupVersionKind(),
	}
	if opts != nil {
		statusHandler.opts = *opts
	}
	RegisterUserStatusHandler(ctx, controller, condition, name, statusHandler.Handle)
}

type userStatusHandler struct {
	client    UserClient
	condition condition.Cond
	handler   UserStatusHandler
}

func (a *userStatusHandler) sync(key string, obj *v3.User) (*v3.User, error) {
	if obj == nil {
		return obj, nil
	}

	origStatus := obj.Status
	obj = obj.DeepCopy()
	newStatus, err := a.handler(obj, obj.Status)
	if err != nil {
		// Revert to old status on error
		newStatus = *origStatus.DeepCopy()
	}

	obj.Status = newStatus
	if a.condition != "" {
		if errors.IsConflict(err) {
			a.condition.SetError(obj, "", nil)
		} else {
			a.condition.SetError(obj, "", err)
		}
	}
	if !equality.Semantic.DeepEqual(origStatus, obj.Status) {
		var newErr error
		obj, newErr = a.client.UpdateStatus(obj)
		if err == nil {
			err = newErr
		}
	}
	return obj, err
}

type userGeneratingHandler struct {
	UserGeneratingHandler
	apply apply.Apply
	opts  generic.GeneratingHandlerOptions
	gvk   schema.GroupVersionKind
	name  string
}

func (a *userGeneratingHandler) Handle(obj *v3.User, status v3.UserStatus) (v3.UserStatus, error) {
	objs, newStatus, err := a.UserGeneratingHandler(obj, status)
	if err != nil {
		return newStatus, err
	}

	apply := a.apply

	if !a.opts.DynamicLookup {
		apply = apply.WithStrictCaching()
	}

	if !a.opts.AllowCrossNamespace && !a.opts.AllowClusterScoped {
		apply = apply.WithSetOwnerReference(true, false).
			WithDefaultNamespace(obj.GetNamespace()).
			WithListerNamespace(obj.GetNamespace())
	}

	if !a.opts.AllowClusterScoped {
		apply = apply.WithRestrictClusterScoped()
	}

	return newStatus, apply.
		WithOwner(obj).
		WithSetID(a.name).
		ApplyObjects(objs...)
}
